@@ -3,6 +3,8 @@ import collections
 import csv
 import numpy as np
 import tensorflow as tf
+from tensorflow.python.data.ops.dataset_ops import ConcatenateDataset
+
 from tensorflow_datasets.core import api_utils
 import tensorflow_datasets as tfds
 from tensorflow.python import feature_column
@@ -261,8 +263,11 @@ class SuperDatasetBuilder(tfds.core.GeneratorBasedBuilder):
         datasets, data_info = tfds.load(dataset_name,
                                     split=split,
                                     as_supervised=True,
-                                    with_info=True)
-        batched_datasets = []
+                                    with_info=True,
+                                    as_dataset_kwargs=dict(
+                                        shuffle_files=False,
+                                    ))
+        batched_datasets = []  # 对应split的数据集个数
         for dataset in datasets:
             if map_fn_feature_label:
                 dataset = dataset.map(map_fn_feature_label)
@@ -270,10 +275,17 @@ class SuperDatasetBuilder(tfds.core.GeneratorBasedBuilder):
                 batched_datasets.append(dataset.batch(batch_size).prefetch(prefetch).repeat(epochs))
             else:
                 batched_datasets.append(dataset.shuffle(shuffle).batch(batch_size).prefetch(prefetch).repeat(epochs))
-        if len(batched_datasets) == 1:
-            # 1个元素的列表就直接返回该元素
-            batched_datasets = batched_datasets[0]
-        return batched_datasets
+        # if len(batched_datasets) == 1:
+        #     # 1个元素的列表就直接返回该元素
+        #     batched_datasets = batched_datasets[0]
+        # return batched_datasets
+        concated_batched_datasets = None
+        for data in batched_datasets:
+            if concated_batched_datasets is None:
+                concated_batched_datasets = data
+            else:
+                concated_batched_datasets = concated_batched_datasets.concatenate(data)
+        return concated_batched_datasets
 
     @staticmethod
     def read_batched_datasets(batched_datasets):
@@ -291,7 +303,7 @@ class SuperDatasetBuilder(tfds.core.GeneratorBasedBuilder):
             yield features, labels
 
     @staticmethod
-    def split_full_datasets(full_datasets:tf.data.Dataset, num_examples=0, train_size=0.7, validation_size=0, test_size=0.3):
+    def split_full_datasets(full_datasets: tf.data.Dataset, num_examples=0, train_size=0.7, validation_size=0, test_size=0.3):
         """
         num_examples=total num examples | total num batched
         ```
@@ -304,17 +316,44 @@ class SuperDatasetBuilder(tfds.core.GeneratorBasedBuilder):
                                                                                                   test_size=0.2)
         ```
         """
+        train_shard_num = int(train_size*10)
         train_size = int(train_size * num_examples)
+        validation_shard_num = int(validation_size*10)
         validation_size = int(validation_size * num_examples)
+        test_shard_num = int(test_size*10)
         test_size = int(test_size * num_examples)
-        train_dataset = full_datasets.take(train_size)
-        validation_dataset = full_datasets.skip(train_size).take(validation_size)
-        test_dataset = full_datasets.skip(train_size).skip(validation_size).take(test_size)
+
+        # 注意：用skip/take方法取validationh和test数据集很慢
+        # train_dataset = full_datasets.take(train_size)
+        # validation_dataset = full_datasets.skip(train_size).take(validation_size)
+        # test_dataset = full_datasets.skip(train_size).skip(validation_size).take(test_size)
+
+        # 注意：用skip/take方法取validationh和test数据集很慢
+        # train_dataset = full_datasets.take(train_size)
+        # test_dataset = full_datasets.skip(train_size)
+        # validation_dataset = test_dataset.skip(validation_size)
+        # test_dataset = test_dataset.take(test_size)
+        # 用shard方法获取数据集比较快，注意要确保shard之后再去做shuffle的操作
+        train_dataset = full_datasets.take(0)
+        validation_dataset = full_datasets.take(0)
+        test_dataset = full_datasets.take(0)
+        for i in range(train_shard_num):
+            train_dataset = train_dataset.concatenate(full_datasets.shard(10, i))
+        for i in range(train_shard_num, train_shard_num + validation_shard_num):
+            validation_dataset = validation_dataset.concatenate(full_datasets.shard(10, i))
+        for i in range(train_shard_num + validation_shard_num, train_shard_num + validation_shard_num + test_shard_num):
+            test_dataset = test_dataset.concatenate(full_datasets.shard(10, i))
         return train_dataset, validation_dataset, test_dataset
 
     @staticmethod
     def camelcase_to_snakecase(class_name):
-        """Convert camel-case string to snake-case."""
+        """
+        Convert camel-case string to snake-case.
+        usage:
+        ```
+        camelcase_to_snakecase(MovieLensDatasetBuilder.__name__)
+        ```
+        """
         return CommonUtils.camelcase_to_snakecase(class_name)
 
     @staticmethod
@@ -336,6 +375,10 @@ class SuperDatasetBuilder(tfds.core.GeneratorBasedBuilder):
         """dataset_builder_cls_name: mnist/plain_text 或 mnist"""
         return tfds.builder(dataset_builder_cls_name).info
 
+    @staticmethod
+    def get_split_name_list(dataset_builder_cls_name=None):
+        return list(tfds.builder(dataset_builder_cls_name).info.splits.keys())
+
     @staticmethod
     def build_all_numeric_feature_columns(dataset_builder_cls_name=None, supervised_keys_feature_name="feature"):
         """
